Class: Sequent::Core::Workflow
- Inherits:
-
Object
- Object
- Sequent::Core::Workflow
show all
- Extended by:
- ActiveSupport::DescendantsTracker
- Includes:
- Helpers::MessageHandler
- Defined in:
- lib/sequent/core/workflow.rb
Class Method Summary
collapse
Instance Method Summary
collapse
#handle_message, included
Class Method Details
.on(*args, **opts, &block) ⇒ Object
12
13
14
15
16
17
18
19
20
21
22
23
|
# File 'lib/sequent/core/workflow.rb', line 12
def self.on(*args, **opts, &block)
decorated_block = ->(event) do
begin
old_event = CurrentEvent.current
CurrentEvent.current = event
instance_exec(event, &block)
ensure
CurrentEvent.current = old_event
end
end
super(*args, **opts, &decorated_block)
end
|
Instance Method Details
#after_commit(ignore_errors: false, &block) ⇒ Object
Workflow#after_commit will accept a block to execute after the transaction commits. This is very useful to isolate side-effects. They will run only on the transaction’s success and will not be able to roll it back when there is an exception. Useful if your background jobs processor is not using the same database connection to enqueue jobs.
36
37
38
39
40
41
42
43
44
|
# File 'lib/sequent/core/workflow.rb', line 36
def after_commit(ignore_errors: false, &block)
Sequent.configuration.transaction_provider.after_commit(&block)
rescue StandardError => e
if ignore_errors
Sequent.logger.warn("An exception was raised in an after_commit hook: #{e}, #{e.inspect}")
else
raise e
end
end
|
#execute_commands(*commands) ⇒ Object
25
26
27
|
# File 'lib/sequent/core/workflow.rb', line 25
def execute_commands(*commands)
Sequent.configuration.command_service.execute_commands(*commands)
end
|